<?php
/* vim: set ts=4 sw=4 sts=4 et: */
/*****************************************************************************\
+-----------------------------------------------------------------------------+
| X-Cart Software license agreement                                           |
| Copyright (c) 2001-present Qualiteam software Ltd <info@x-cart.com>         |
| All rights reserved.                                                        |
+-----------------------------------------------------------------------------+
| PLEASE READ  THE FULL TEXT OF SOFTWARE LICENSE AGREEMENT IN THE "COPYRIGHT" |
| FILE PROVIDED WITH THIS DISTRIBUTION. THE AGREEMENT TEXT IS ALSO AVAILABLE  |
| AT THE FOLLOWING URL: https://www.x-cart.com/license-agreement-classic.html |
|                                                                             |
| THIS AGREEMENT EXPRESSES THE TERMS AND CONDITIONS ON WHICH YOU MAY USE THIS |
| SOFTWARE PROGRAM AND ASSOCIATED DOCUMENTATION THAT QUALITEAM SOFTWARE LTD   |
| (hereinafter referred to as "THE AUTHOR") OF REPUBLIC OF CYPRUS IS          |
| FURNISHING OR MAKING AVAILABLE TO YOU WITH THIS AGREEMENT (COLLECTIVELY,    |
| THE "SOFTWARE"). PLEASE REVIEW THE FOLLOWING TERMS AND CONDITIONS OF THIS   |
| LICENSE AGREEMENT CAREFULLY BEFORE INSTALLING OR USING THE SOFTWARE. BY     |
| INSTALLING, COPYING OR OTHERWISE USING THE SOFTWARE, YOU AND YOUR COMPANY   |
| (COLLECTIVELY, "YOU") ARE ACCEPTING AND AGREEING TO THE TERMS OF THIS       |
| LICENSE AGREEMENT. IF YOU ARE NOT WILLING TO BE BOUND BY THIS AGREEMENT, DO |
| NOT INSTALL OR USE THE SOFTWARE. VARIOUS COPYRIGHTS AND OTHER INTELLECTUAL  |
| PROPERTY RIGHTS PROTECT THE SOFTWARE. THIS AGREEMENT IS A LICENSE AGREEMENT |
| THAT GIVES YOU LIMITED RIGHTS TO USE THE SOFTWARE AND NOT AN AGREEMENT FOR  |
| SALE OR FOR TRANSFER OF TITLE. THE AUTHOR RETAINS ALL RIGHTS NOT EXPRESSLY  |
| GRANTED BY THIS AGREEMENT.                                                  |
+-----------------------------------------------------------------------------+
\*****************************************************************************/

/**
 * cURL HTTPS module
 *
 * @category   X-Cart
 * @package    X-Cart
 * @subpackage Lib
 * @author     Ruslan R. Fazlyev <rrf@x-cart.com>
 * @copyright  Copyright (c) 2001-present Qualiteam software Ltd <info@x-cart.com>
 * @license    https://www.x-cart.com/license-agreement-classic.html X-Cart license agreement
 * @version    cf9e608d41c40f761c6416f642a1d0094a6af214, v37 (xcart_4_7_7), 2017-01-24 09:29:34, func.https_curl.php, aim
 * @link       http://www.x-cart.com/
 * @see        ____file_see____
 */

if (!defined('XCART_START')) { header("Location: ../"); die("Access denied"); }

// INPUT:
//
// $method          [string: POST|GET]
//
// $url             [string]
//    www.yoursite.com:443/path/to/script.asp
//
// $data            [array]
//    $data[] = "parametr=value";
//
// $join            [string]
//    $join = "\&";
//
// $cookie          [array]
//    $cookie = "parametr=value";
//
// $conttype        [string]
//    $conttype = 'text/xml';
//
// $referer         [string]
//    $conttype = "http://www.yoursite.com/index.htm";
//
// $cert            [string]
//    $cert = "../certs/demo-cert.pem";
//
// $kcert           [string]
//    $keyc = "../certs/demo-keycert.pem";
//
// $rhead           [string]
//    $rhead = '...';
//
// $rbody           [string]
//    $rbody = '...';

function func_https_request_curl($method, $url, $data="", $join="&", $cookie="", $conttype="application/x-www-form-urlencoded", $referer="", $cert="", $kcert="", $headers="", $timeout = 0, $use_tls = false)
{
    global $config;
    global $xcart_dir;

    if ($method != 'POST' && $method != 'GET' && $method != 'DELETE')
        return array('0',"X-Cart HTTPS: Invalid method");

    if (!preg_match("/^(https?:\/\/)(.*\@)?([a-z0-9_\.\-]+):(\d+)(\/.*)$/SUi",$url,$m))
        return array('0',"X-Cart HTTPS: Invalid URL");

    $curl_binary = func_find_executable('curl');
    if (!$curl_binary)
        return array('0',"X-Cart HTTPS: curl executable is not found");

    if (!X_DEF_OS_WINDOWS)
        putenv("LD_LIBRARY_PATH=".getenv('LD_LIBRARY_PATH').":".dirname($curl_binary));

    $tmpfile = func_temp_store('');

    if (empty($tmpfile))
        return array(0, "X-Cart HTTPS: cannot create temporaly file");

    $execline = func_shellquote($curl_binary) . (func_constant('HTTP_1_0_COMPATIBILITY_MODE') ? ' --http1.0' : '') . ' -D-';
    @exec(func_shellquote($curl_binary)." --version", $output);
    $version = @$output[0];
    // -k|--insecure key is supported by curl since version 7.10
    $supports_insecure = false;
    if (preg_match('/curl ([^ $]+)/S', $version, $m) ){
        $parts = explode('.',$m[1]);
        if( $parts[0] > 7 || ($parts[0] == 7 && $parts[1] >= 10) )
            $supports_insecure = true;
    }

    if (!empty($config['General']['https_proxy'])) {
        $execline .= " --proxy ".$config['General']['https_proxy'];
        // uncomment this line if you need proxy tunnel
        // $execline .= " --proxytunnel";
    }

    // Set GET method flag
    if ($method=="GET") {
        $execline.= " --get";
    } elseif ($method == "DELETE") {
        $execline.= " --request " . $method;
    }

    // Set TimeOut parameter
    $timeout = abs(intval($timeout));
    if (!empty($timeout)) {
        $execline.= " --connect-timeout ".$timeout." -m ".$timeout;
    }

    // Combine REQUEST string
    $request_file = false;
    if ($data) {
        if ($join && is_array($data)) {
            foreach($data as $k => $v) {
                list($a, $b) = explode("=", trim($v), 2);
                $data[$k] = $a."=".urlencode($b);
            }
        }

        $request_file = func_temp_store(is_array($data) ? join($join, $data) : $data);
        $execline .= " -d ".func_shellquote('@'.$request_file);
    }

    // Add SSL Certificate
    if ($cert) {
        $execline.= " --cert ".func_shellquote($cert);

        // Add SSL Key-Certificate
        if ($kcert)
            $execline.= " --key ".func_shellquote($kcert);
    }

    if (
        defined('USE_CURLOPT_SSL_VERIFYPEER')
        && file_exists($xcart_dir . '/payment/certs/curl-ca-bundle.crt')
    ) {
        $execline.= " --cacert '$xcart_dir/payment/certs/curl-ca-bundle.crt'";
    } elseif ($supports_insecure ) {
        $execline.= " -k ";
    }

    if ($cookie)
        $execline.=" --cookie ".func_shellquote(join(';',$cookie));

    // Add Content-Type...
    if ($conttype != "application/x-www-form-urlencoded") {
        $execline.=" -H ".func_shellquote('Content-Type: '.$conttype);
    }

    // Add referer
    if ($referer != '') {
        $execline.=" -H ".func_shellquote('Referer: '.$referer);
    }

    if (empty($headers)) {
        $headers = array();
    }

    // Additional headers
    if (is_array($headers)) {

        if (!func_constant('HTTP_1_0_COMPATIBILITY_MODE') && !isset($headers['Connection'])) {
            // Always close HTTP 1.1 connection, because all code designed for HTTP1.0-like connections
            $headers['Connection'] = 'close';
        }

        foreach ($headers as $k=>$v) {
            if (is_integer($k)) {
                $execline .= " -H \"".addslashes($v)."\"";
            }
            else {
                $execline .= " -H \"$k: ".addslashes($v)."\"";
            }
        }
    }

    if ($use_tls) {
        $execline .= ' --tlsv1';
    }

    $fp = popen($execline." ".func_shellquote($url)." 2>".func_shellquote($tmpfile), 'r');
    if (!$fp) {
        @unlink($tmpfile);
        return array(0, "X-Cart HTTPS: curl execution failed");
    }

    //$fp = str_ireplace("HTTP/1.0 200 Connection established\r\n\r\n", '', $fp);
    $res = func_https_receive_result($fp);

    pclose($fp);
    @unlink($tmpfile);
    if ($request_file !== false)
        @unlink($request_file);

    func_https_ctl('PUT', $res);

    return $res;
}

?>
